import { Button, ConfigProvider, Form, Input } from "antd";
import { useTranslation } from "@tavi/i18n";

import {
  AiOutlineUser,
  AiOutlineMobile,
  AiOutlineLock,
  AiOutlineSafetyCertificate,
} from "react-icons/ai";
import { REG } from "@tavi/util";

export interface FormFields {
  username?: string;
  password?: string;
  phoneNumber?: number;
  verifyCode?: number;
}
interface LoginFormProps {
  children?: JSX.Element;
  styles?: React.CSSProperties;
  onFormChange?: (v: FormFields) => void;
  onLogin: (v: FormFields) => void;
}

export const LoginForm = (props: LoginFormProps) => {
  const [form] = Form.useForm();
  const { t } = useTranslation();

  const handleLogin = async () => {
    try {
      await form.validateFields();
      props.onLogin(form.getFieldsValue());
    } catch (error) {
      console.log(error);
    }
  };

  const handleFormChange = () => {
    if (props.onFormChange) props?.onFormChange(form.getFieldsValue());
  };

  return (
    <Form
      className="login-form-group"
      layout="vertical"
      onChange={handleFormChange}
      form={form}
      style={{ ...props.styles }}
    >
      <ConfigProvider
        theme={{
          token: {
            colorPrimary: "#585963",
            colorBorder: "#585963",
            colorBgBase: "transparent",
            colorTextBase: "#fff",
          },
        }}
      >
        <Form.Item
          label={t("LOGIN.FORM_ITEM.LABEL_1")}
          name="username"
          rules={[
            {
              required: true,
              min: 6,
              max: 20,
              message: "请输入用户名",
            },
          ]}
        >
          <Input
            size="large"
            prefix={<AiOutlineUser />}
            placeholder={t("LOGIN.FORM_ITEM.LABEL_5") as string}
            onPressEnter={handleLogin}
            autoComplete="off"
          />
        </Form.Item>
        <Form.Item
          label={t("LOGIN.FORM_ITEM.LABEL_2")}
          name="password"
          rules={[
            {
              required: true,
              pattern: REG.password,
              min: 6,
              max: 20,
              message: "6-20位，包含 大写、小写、字母、特殊字符",
            },
          ]}
        >
          <Input.Password
            size="large"
            prefix={<AiOutlineLock />}
            placeholder={t("LOGIN.FORM_ITEM.LABEL_6") as string}
            onPressEnter={handleLogin}
            autoComplete="off"
          />
        </Form.Item>
        <Form.Item
          label={t("LOGIN.FORM_ITEM.LABEL_3")}
          name="phoneNumber"
          rules={[
            {
              required: true,
              pattern: REG.phoneNumber,
              min: 11,
              max: 11,
              message: "请输入正确的手机号",
            },
          ]}
        >
          <Input
            size="large"
            prefix={<AiOutlineMobile />}
            placeholder={t("LOGIN.FORM_ITEM.LABEL_7") as string}
            onPressEnter={handleLogin}
            autoComplete="off"
          />
        </Form.Item>
        <Form.Item
          label={t("LOGIN.FORM_ITEM.LABEL_4")}
          name="verifyCode"
          style={{ marginBottom: "4.29rem" }}
          rules={[
            {
              min: 6,
              max: 6,
              message: "请输入验证码",
            },
          ]}
        >
          <Input
            size="large"
            prefix={<AiOutlineSafetyCertificate />}
            placeholder={t("LOGIN.FORM_ITEM.LABEL_8") as string}
            onPressEnter={handleLogin}
            autoComplete="off"
          />
        </Form.Item>
      </ConfigProvider>
      <ConfigProvider
        theme={{
          token: {
            colorPrimary: "#2E97D1",
            colorTextBase: "#fff",
          },
        }}
      >
        <Form.Item style={{ marginBottom: 0 }}>
          <Button size="large" block type="primary" onClick={handleLogin}>
            {t("LOGIN.FORM_ITEM.BUTTON_1")}
          </Button>
        </Form.Item>
      </ConfigProvider>
    </Form>
  );
};
