import { CreateUserFormFields } from "@/modules/User/List/CreateUserForm";
import { Permission } from "./entities/Permission";
import { RbacRepository } from "./RbacRepository";
import { makeAutoObservable } from "mobx";
import { Role } from "./entities/Role";
import { User } from "./entities/User";

export class RbacService {
  roleList: Role[];
  allPermissions: Permission[];
  allRoles: Role[];
  activeRole: Role | undefined;
  userList: User[];
  activeUser: User | undefined;
  constructor(private rbacRepository: RbacRepository) {
    this.roleList = [];
    this.allPermissions = [];
    this.allRoles = [];
    this.userList = [];
    makeAutoObservable(this);
  }

  /**
   * 查询全部用户
   */
  async findAllRoleWithPermissions() {
    const { allPermissions, roleWithPermissions } =
      await this.rbacRepository.findAllRoleWithPermissions();
    this.allPermissions = allPermissions;
    this.roleList = roleWithPermissions;
    this.activeRole = roleWithPermissions[0];
    this.allRoles = roleWithPermissions;
    return roleWithPermissions;
  }

  /**
   * 添加角色
   */
  async addRole(role: Role) {
    this.roleList = [
      ...this.roleList,
      { ...role, isEnabled: true, permissions: [] },
    ];
    return await this.rbacRepository.saveRole(role);
  }

  /**
   * 删除角色
   */
  async removeRole(role: Role) {
    this.roleList = this.roleList.filter((r) => r.id !== role.id);
    return await this.rbacRepository.removeRole(role);
  }

  /**
   * 编辑角色
   */
  async updateRole(role: Role) {
    const success = await this.rbacRepository.updateRole(role);
    if (success)
      this.roleList = this.roleList.map((r) => (r.id === role.id ? role : r));
    return success;
  }

  /**
   * 查询角色拥有的权限
   * 返回：全部权限、已有权限
   */
  async findRolePermissions(id: number) {
    return await this.rbacRepository.findRolePermissions(id);
  }

  /**
   * 更新某个角色的权限
   * @param {string[]} permissionIds 权限id字符串数组
   * @param {number} roleId 角色id
   */
  async updateRolePermissions(permissionIds: string[], roleId: number) {
    await this.rbacRepository.saveRolePermissions(permissionIds, roleId);
  }

  /**
   * 设置当前选中的角色
   */
  setActiveRole(role: Role) {
    this.activeRole = this.roleList.find((r) => r.id === role.id);
  }

  /**
   * 查询全部用户
   */
  async findAllUser() {
    const users = await this.rbacRepository.findAllUser();
    this.userList = users;
    return users;
  }

  /**
   * 创建用户
   */
  async saveUser(fields: CreateUserFormFields) {
    const { roleIds, username, password, phoneNumber } = fields;
    const roleMapping = this.allRoles.map((r) => ({
      alias: r.alias,
      description: r.description,
      id: r.id,
      isEnabled: r.isEnabled,
      name: r.name,
    }));
    const user = {
      id: Date.now(),
      roles: roleMapping.filter((r) => roleIds?.includes(r.id!)),
      isEnabled: true,
      username,
      phoneNumber,
      createTime: new Date().toLocaleString(),
    };
    this.userList = [...this.userList, user];
    return await this.rbacRepository.saveUser({ ...user, password });
  }

  /**
   * 修改用户角色、手机号、密码、启用/封禁
   */
  async updateUser(user: User, roleIds: number[]) {
    const { password, phoneNumber, isEnabled, roles, id, username } = user;
    const success = await this.rbacRepository.updateUser({
      password,
      phoneNumber,
      isEnabled,
      roles: roleIds,
      id,
      username,
    });
    this.userList = this.userList.map((u) => {
      if (u.id === id) {
        u.phoneNumber = phoneNumber;
        u.isEnabled = isEnabled;
        u.roles = roles;
      }
      return u;
    });
    return success;
  }

  async deleteUser(userIds: number[]) {
    const success = await this.rbacRepository.deleteUser(userIds);
    this.userList = this.userList.filter((u) => !userIds.includes(u.id!));
    return success;
  }
}
