import axios, {
  AxiosInstance,
  AxiosRequestConfig,
  AxiosResponse,
  CancelTokenSource,
  InternalAxiosRequestConfig,
} from "axios";

interface RequestConfig extends InternalAxiosRequestConfig {
  onRequestSent?: RequestCallback;
}

type RequestCallback = (token: string) => void;

class AxiosRequestInstance {
  private instance: AxiosInstance;
  private cancelTokenMap: Map<string, CancelTokenSource>;

  constructor() {
    this.instance = axios.create();
    this.cancelTokenMap = new Map();

    // 请求拦截
    this.instance.interceptors.request.use((config: RequestConfig) => {
      const token = this.generateToken();
      const source = axios.CancelToken.source();
      this.cancelTokenMap.set(token, source);

      // 生成 traceId 并加入请求头
      const traceId = this.generateTraceId();
      config.headers["x-trace-id"] = traceId;
      config.headers["x-request-token"] = token; // 将token加入请求头
      config.cancelToken = source.token;

      if (config.onRequestSent) config.onRequestSent(token);
      return config;
    });

    // 响应拦截
    this.instance.interceptors.response.use((response: AxiosResponse) => {
      const token = response.headers["x-request-token"];
      this.cancelTokenMap.delete(token); // 完成后删除此token
      return response;
    });
  }

  // 生成token
  private generateToken(): string {
    return Math.random().toString(36).substring(2);
  }

  // 生成traceId
  private generateTraceId(): string {
    // 使用你自己的 traceId 生成策略，这里是一个简单的示例
    return Math.random().toString(36).substring(2);
  }

  // 取消请求
  public cancelRequest(token: string): void {
    const source = this.cancelTokenMap.get(token);
    if (source) {
      source.cancel("Request canceled, token:" + token);
      this.cancelTokenMap.delete(token);
    }
  }

  // 封装get、post等请求
  public async get<T = any>(
    url: string,
    config?: AxiosRequestConfig & { onRequestSent?: RequestCallback }
  ): Promise<T> {
    const response = await this.instance.get<T>(url, config);
    return response.data;
  }

  public async post<T = any>(
    url: string,
    data?: any,
    config?: RequestConfig & { onRequestSent?: RequestCallback }
  ): Promise<T> {
    const response = await this.instance.post<T>(url, data, config);
    return response.data;
  }

  public async put<T = any>(
    url: string,
    data?: any,
    config?: RequestConfig & { onRequestSent?: RequestCallback }
  ): Promise<T> {
    const response = await this.instance.put<T>(url, data, config);
    return response.data;
  }

  public async delete<T = any>(
    url: string,
    config?: RequestConfig & { onRequestSent?: RequestCallback }
  ): Promise<T> {
    const response = await this.instance.delete<T>(url, config);
    return response.data;
  }
}

export const Request = new AxiosRequestInstance();

// 使用方法:
// 1. 发起请求并获取token
// let token: string;
// requestInstance.get("https://api.example.com/data", {
//   onRequestSent: (requestToken) => {
//     token = requestToken;
//   },
// });

// 2. 在需要取消请求的时候取消请求
// requestInstance.cancelRequest(token);
