import { useDomain } from "@/hook/useDomain";
import { Button, Space, Spin, Table, Tooltip, message } from "antd";
import { useEffect, useState } from "react";
import { columnsForStudy, columnsForSeries } from "./columns";
import { DesktopOutlined, FileZipOutlined } from "@ant-design/icons";
import { openOHIFViewer } from "../Upload/util";

interface DicomListProps {
  children?: JSX.Element;
}

export type SeriesItemType = {
  BodyPartExamined: string;
  ContrastBolusAgent: string;
  ID: string;
  ImageOrientationPatient: string;
  Instances: string[];
  Manufacturer: string;
  Modality: string;
  OperatorsName: string;
  ProtocolName: string;
  SeriesDate: string;
  SeriesDescription: string;
  SeriesInstanceUID: string;
  SeriesNumber: string;
  SeriesTime: string;
  StationName: string;
  Type: "study";
  tags: any;
};

export type StudyItemType = {
  AccessionNumber: string;
  ID: string;
  InstitutionName: string;
  PatientBirthDate: string;
  PatientID: string;
  PatientName: string;
  PatientSex: string;
  ReferringPhysicianName: string;
  StudyDate: string;
  StudyID: string;
  StudyInstanceUID: string;
  StudyTime: string;
  Type: "series";
  subs: SeriesItemType[];
};

export const DicomList = (props: DicomListProps) => {
  const [dataSource, setDataSource] = useState<StudyItemType[]>([]);
  const [tableLoading, setTableLoading] = useState(false);
  const [loading, setLoading] = useState(false);
  const [messageApi, contextHolder] = message.useMessage();
  const { userDomainService, dicomDomainService } = useDomain();
  useEffect(() => {
    setTableLoading(true);
    userDomainService.findDicoms().then((result) => {
      const { data } = result;
      setDataSource(data as StudyItemType[]);
      setTableLoading(false);
    });
  }, []);

  /**
   * 序列级别
   */
  const expandedRowRenderForSeries = (record: StudyItemType) => {
    return (
      <Table
        rowKey="SeriesInstanceUID"
        columns={[
          ...columnsForSeries,
          {
            title: "操作",
            dataIndex: "operation",
            render: (_: any, recordSeries: SeriesItemType) => (
              <Space>
                <Tooltip title="在线阅片">
                  <Button
                    type="text"
                    icon={<DesktopOutlined />}
                    onClick={() => onViewDicom(record, recordSeries)}
                  />
                </Tooltip>
                <Tooltip title="下载原始影像 .zip文件">
                  <Button
                    type="text"
                    icon={<FileZipOutlined />}
                    onClick={() => onDownloadDicom(recordSeries)}
                  />
                </Tooltip>
              </Space>
            ),
          },
        ]}
        dataSource={record.subs}
        pagination={false}
      />
    );
  };

  const onViewDicom = (record: StudyItemType, recordSeries: SeriesItemType) => {
    const { StudyInstanceUID } = record;
    const { SeriesInstanceUID } = recordSeries;
    openOHIFViewer(StudyInstanceUID, SeriesInstanceUID);
  };

  const onDownloadDicom = ({ ID, Type }: SeriesItemType | StudyItemType) => {
    setLoading(true);
    dicomDomainService.downloadDicom({ ID, Type }).finally(() => {
      messageApi.success("数据下载成功");
      setLoading(false);
    });
  };

  return (
    <div>
      {contextHolder}
      {loading && (
        <div
          style={{
            position: "fixed",
            background: "rgb(255 195 195 / 30%)",
            left: 0,
            right: 0,
            top: 0,
            bottom: 0,
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            zIndex: 19940121,
          }}
        >
          <Spin size="large" />
        </div>
      )}
      <Table
        dataSource={dataSource}
        columns={[
          ...columnsForStudy,
          {
            title: "操作",
            dataIndex: "operation",
            render: (_: any, record: StudyItemType) => (
              <Space>
                <Tooltip title="下载原始影像 .zip文件">
                  <Button
                    type="text"
                    icon={<FileZipOutlined />}
                    onClick={() => onDownloadDicom(record)}
                  />
                </Tooltip>
              </Space>
            ),
          },
        ]}
        loading={tableLoading}
        expandable={{
          expandedRowRender: expandedRowRenderForSeries,
        }}
        rowKey="StudyInstanceUID"
      />
    </div>
  );
};
