import { EventPattern } from '@nestjs/microservices';
import { RedisService } from './redis/redis.service';
import { ConfigService } from '@nestjs/config';
import { Controller } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';

@Controller()
export class AppController {
  constructor(
    private configService: ConfigService,
    private redisService: RedisService,
    private jwtService: JwtService,
  ) {}

  /**
   * 给业务系统登录接口进行token签发,注入用户的角色信息
   * @description redis的EX、cookie的maxAge(ms)、jwt的expiresIn，三者保持一致
   */
  @EventPattern({ cmd: 'cert.token.create' })
  async createToken(payload) {
    const { username } = payload;
    const token = await this.jwtService.sign(payload);
    this.redisService.setToken(username, token);
    return { token };
  }

  /**
   * 检查token有效性 && 解token
   * @description 业务系统的guard对请求token进行拦截，校验每次请求token是否合法&是否在redis中
   */
  @EventPattern({ cmd: 'cert.token.decode' })
  async decodeToken(
    token: string,
  ): Promise<{ tokenValid: boolean; payload: unknown; error?: unknown }> {
    try {
      await this.jwtService.verify(token);
      const payload = (await this.jwtService.decode(token)) as any;
      const { username } = payload;
      const inRedis = await this.redisService.tokenInRedis(username);
      return { tokenValid: true && inRedis, payload };
    } catch (error) {
      return { tokenValid: false, payload: null, error };
    }
  }

  /**
   * 获取token在cookie中存储的key
   * @description cookie的maxAge单位是毫秒，如果给cookie使用expires需要 x1000
   */
  @EventPattern({ cmd: 'cert.token.config' })
  async tokenConfig() {
    return {
      tokenResignIn: this.configService.get('TOKEN_RESIGN_IN'),
      tokenKeyInCookie: this.configService.get('TOKEN_KEY_IN_COOKIE'),
      expires: Number(this.configService.get('JWT_EXPIRES_IN')),
    };
  }

  @EventPattern({ cmd: 'cert.token.deprecated' })
  async removeToken(payload) {
    const { username } = payload;
    console.log('username', username);
    return await this.redisService.removeToken(username);
  }
}
