import { Controller } from '@nestjs/common';
import { EventPattern } from '@nestjs/microservices';
import { UserService } from './user.service';
import { BcryptService } from '../bcrypt/bcrypt.service';

@Controller()
export class UserController {
  constructor(
    private readonly userService: UserService,
    private readonly bcryptService: BcryptService,
  ) {}
  /**
   * 用户账号、密码、是否可用
   */
  @EventPattern({ cmd: 'cert.user.account' })
  async findUser(
    payload,
  ): Promise<{ isLegal: boolean; msg?: string; data?: any }> {
    const { username, password } = payload;
    const user = await this.userService.findUser({ username });
    if (!user) return { isLegal: false, msg: '用户不存在' };
    if (!user.isEnabled) return { isLegal: false, msg: '用户被封禁' };
    const correctPass = await this.bcryptService.comparePassword(
      password,
      user.password,
    );
    if (!correctPass) return { isLegal: false, msg: '密码错误' };
    return { isLegal: true, data: user };
  }

  @EventPattern({ cmd: 'cert.user.encrypt' })
  async encrypt(plainText: string) {
    return await this.bcryptService.hashPassword(plainText);
  }

  @EventPattern({ cmd: 'cert.encrypt.compare' })
  async compare(plainText: string, hash: string) {
    return await this.bcryptService.comparePassword(plainText, hash);
  }

  @EventPattern({ cmd: 'cert.user.create' })
  async createUser(payload) {
    return await this.userService.create(payload);
  }

  @EventPattern({ cmd: 'cert.user.update' })
  async updateUser(payload) {
    const { password, ...rest } = payload;
    const updateFields = password
      ? { password: await this.bcryptService.hashPassword(password), ...rest }
      : { ...rest };
    return await this.userService.update(updateFields);
  }

  @EventPattern({ cmd: 'cert.user.delete' })
  async deleteUser(userIds: number[]) {
    return await this.userService.delete(userIds);
  }

  @EventPattern({ cmd: 'cert.user.find.annotator' })
  async findAnnotators() {
    const annotators = await this.userService.findAnnotators();
    return { data: annotators };
  }
}
