import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { User } from '../rbac/entity/user.entity';
import { In, Repository } from 'typeorm';
import { Role } from '../rbac/entity/role.entity';
import { BcryptService } from '../bcrypt/bcrypt.service';

@Injectable()
export class UserService {
  constructor(
    @InjectRepository(User) private readonly userRepository: Repository<User>,
    @InjectRepository(Role) private readonly roleRepository: Repository<Role>,
    private readonly bcryptService: BcryptService,
  ) {}

  async findAnnotators() {
    const annotators = await this.userRepository
      .createQueryBuilder('user')
      .select(['user.id', 'user.username'])
      .leftJoin('user.roles', 'role')
      .addSelect('role.name')
      .where('role.name = :roleName', { roleName: 'dmp_annotator' })
      .andWhere('user.isEnabled = :isEnabled', { isEnabled: true })
      .getMany();
    return annotators;
  }

  async findUser({ username }) {
    return await this.userRepository.findOne({
      where: { username },
      select: ['id', 'isEnabled', 'phoneNumber', 'password'],
      relations: ['roles'],
    });
  }

  async create(payload: any) {
    const { username, password, phoneNumber, roles: selectRoles } = payload;
    // 判断用户名是否存在
    const user = await this.userRepository.findOne({ where: { username } });
    const roles = await this.roleRepository.find({
      where: {
        id: In(selectRoles.map((r) => r.id)),
      },
    });
    // 如果username存在
    if (user) return false;
    const cipher = await this.bcryptService.hashPassword(password);
    return await this.userRepository.save({
      username,
      password: cipher,
      phoneNumber,
      roles,
    });
  }

  async update(fields: User) {
    try {
      const { id, roles: roleIds } = fields;
      const user = await this.userRepository.findOne({
        where: { id },
        relations: ['roles'],
      });
      const roles = await this.roleRepository.find({
        where: {
          id: In(roleIds),
        },
      });
      const updateUser = { ...user, ...fields, roles };
      const result = await this.userRepository.save(updateUser);
      if (result) return { id: user.id };
    } catch (error) {
      throw new Error(error);
    }
  }

  async delete(userIds: number[]) {
    const users = await this.userRepository.find({
      where: { id: In(userIds) },
      relations: { roles: true },
    });
    users.forEach((user) => (user.roles = []));
    await this.userRepository.save(users);
    return await this.userRepository.remove(users);
  }
}
