import { Controller } from '@nestjs/common';
import { AppService } from './app.service';
import { EventPattern } from '@nestjs/microservices';
import { ConfigService } from '@nestjs/config';
import axios from 'axios';

const metaFormatter = (data) => {
  return {
    StudyDate: data['00080020']?.Value?.[0] ?? '',
    StudyTime: data['00080030']?.Value?.[0] ?? '',
    AccessionNumber: data['00080050']?.Value?.[0] ?? '',
    Modality: data['00080060']?.Value?.[0] ?? '',
    PatientName: (data['00100010']?.Value?.[0] as any).Alphabetic,
    PatientID: data['00100020']?.Value?.[0] ?? '',
    PatientBirthDate: data['00100030']?.Value?.[0] ?? '',
    PatientSex: data['00100040']?.Value?.[0] ?? '',
    StudyInstanceUID: data['0020000D']?.Value?.[0] ?? '',
    SeriesInstanceUID: data['0020000E']?.Value?.[0] ?? '',
    CharacterSet: data['00080005']?.Value?.[0] ?? '',
    SeriesDescription: data['0008103E']?.Value?.[0] ?? '',
    StudyID: data['00200010']?.Value?.[0] ?? '',
    SeriesNumber: data['00200011']?.Value?.[0] ?? '',
    NumberSeriesRelatedInstances: data['00201209']?.Value?.[0] ?? '',
    NumberPatientRelatedInstances: data['00201206']?.Value?.[0] ?? '',
    NumberPatientRelatedSeries: data['00201208']?.Value?.[0] ?? '',
    SliceThickness: data['00180050']?.Value?.[0] ?? '',
  };
};

type StudyResourceType = {
  ID: string;
  MainDicomTags: {
    AccessionNumber: string;
    StudyDate: string;
    StudyDescription: string;
    StudyID: string;
    StudyInstanceUID: string;
    StudyTime: string;
  };
  PatientMainDicomTags: {
    PatientBirthDate: string;
    PatientID: string;
    PatientName: string;
    PatientSex: string;
  };
  Series: string[];
  Type: 'Study';
};

type SeriesResourceType = {
  ExpectedNumberOfInstances: number;
  ID: string;
  Instances: string[];
  MainDicomTags: {
    Manufacturer: string;
    Modality: string;
    NumberOfSlices: string;
    ProtocolName: string;
    SeriesDate: string;
    SeriesDescription: string;
    SeriesInstanceUID: string;
    SeriesNumber: string;
    SeriesTime: string;
    StationName: string;
  };
  ParentStudy: string;
  Status: string;
  Type: 'Series';
};

@Controller()
export class AppController {
  constructor(
    private readonly appService: AppService,
    private readonly configService: ConfigService,
  ) {}

  @EventPattern({ cmd: 'dicom.find.dicom' })
  async findDicoms() {
    const pacsUrl = this.configService.get('PACS_URL');
    try {
      // 全部的study pacs uuid
      const { data: study_uuids } = await axios.get(`${pacsUrl}/studies`);
      const result = [];
      // study
      for (const study_uuid of study_uuids) {
        const studyUrl = `${pacsUrl}/studies/${study_uuid}`;
        const { data: studyResource } = await axios.get(studyUrl);
        const { Series, PatientMainDicomTags, MainDicomTags, ID, Type } =
          studyResource as StudyResourceType;
        const subs = [];
        // series + 第一张 instance 的 tags
        for (const series_uuid of Series) {
          const seriesUrl = `${pacsUrl}/series/${series_uuid}`;
          const { data: seriesResource } = await axios.get(seriesUrl);
          const { MainDicomTags, Instances, ID, Type } =
            seriesResource as SeriesResourceType;
          // 取第一张instance
          const InstanceNumber = Instances.length;
          const firstInstanceUrl = `${pacsUrl}/instances/${Instances[0]}/simplified-tags`;
          const tags =
            Instances.length > 0
              ? await axios.get(firstInstanceUrl).then((d) => d.data)
              : {};
          subs.push({ ...MainDicomTags, Type, InstanceNumber, ID, tags });
        }
        result.push({
          ...PatientMainDicomTags,
          ...MainDicomTags,
          Type,
          ID,
          subs,
        });
      }
      return { data: result };
    } catch (error) {
      throw new Error(`Failed to fetch DICOM data: ${error.message}`);
    }
  }

  @EventPattern({ cmd: 'dicom.archive.url' })
  async archiveUrl({ ID, Type }: { ID: string; Type: string }) {
    const pacsUrl = this.configService.get('PACS_URL');
    const mapping = {
      study: 'studies',
      series: 'series',
    };
    return `${pacsUrl}/${mapping[Type.toLowerCase()]}/${ID}/archive`;
  }
}
