import html2canvas from "html2canvas";
import { jsPDF } from "jspdf";

export interface Dom2Img {
  /**
   * 需要截图的dom元素
   */
  element: HTMLElement;
  /**
   * 清晰度控制
   * @default 4
   */
  scale?: number;
}

/**
 * dom转图片
 * 250ms - 350ms之间，平均 300ms/张图
 */
export const dom2Img = async (p: Dom2Img) => {
  const { element, scale = 4 } = p;
  const canvas = await html2canvas(element, { scale, useCORS: true });
  return canvas.toDataURL("image/png");
};

export interface Dom2PDF {
  /**
   * 需要截图的dom元素，多个elements传入dom的数组
   */
  elements: HTMLElement[];
  /**
   * 清晰度控制，影响导出速度
   * @default 4
   */
  scale?: number;
  /**
   * 方向
   */
  orientation?: "p" | "portrait" | "l" | "landscape";
  /**
   * jspdf目标尺寸
   * @default a4
   */
  format?: "a4" | string | number[];
  /**
   * jspdf单位
   * @default mm
   */
  unit?: "pt" | "px" | "in" | "mm" | "cm" | "ex" | "em" | "pc";
  /**
   * 压缩pdf
   * @default true
   */
  compress?: boolean;
  /**
   * 生成进度
   * @param {number} pageNum 已生成的页码
   * @param {number} totalPage 全部页码
   */
  onProgress?: (pageNum: number, totalPage: number) => void;
}

/**
 * dom转pdf高清，默认开启压缩，compression = FAST
 */
export const dom2PDF = async (p: Dom2PDF): Promise<jsPDF> => {
  const {
    elements,
    scale = 4,
    format = "a4",
    unit = "mm",
    orientation = "p",
    compress = true,
  } = p;
  const pdf = new jsPDF({ orientation, unit, format, compress });
  for (let i = 0; i < elements.length; i++) {
    const imgData = await dom2Img({ element: elements[i], scale });
    const { width, height } = pdf.getImageProperties(imgData);
    const pdfWidth = pdf.internal.pageSize.getWidth();
    const pdfHeight = (height * pdfWidth) / width;
    /**
     * scale=4 + png +  默认情况下，耗时 ≈ 2500ms
     * 1.FAST 耗时 ≈ 1400ms
     * 2.scale=3 耗时 ≈ 850ms
     */
    console.time("fast");
    pdf.addImage(
      imgData,
      "PNG",
      0,
      0,
      pdfWidth,
      pdfHeight,
      `page-${i + 1}`,
      "FAST"
    );
    console.timeEnd("fast");
    if (i !== elements.length - 1) pdf.addPage();
    p.onProgress && p.onProgress(i + 1, elements.length);
  }
  return pdf;
};
